OC.L10N.register(
    "pax_fax",
    {
    "Settings" : "Settings",
    "Pax Fax" : "Pax Fax",
    "Enter either your Phaxio credentials or your Sinch credentials in the fields from below:" : "Enter either your Phaxio credentials or your Sinch credentials in the fields from below:",
    "Phaxio API Key" : "Phaxio API Key",
    "Phaxio API Secret" : "Phaxio API Secret",
    "Phaxio Webhook Token" : "Phaxio Webhook Token",
    "I want to see a notification in Nextcloud when a new fax is received:" : "I want to see a notification in Nextcloud when a new fax is received:",
    "I want to receive a notification to the email address from below, when a new fax is received" : "I want to receive a notification to the email address from below, when a new fax is received",
    "Save" : "Save",
    "New fax" : "New fax",
    "New fax from Pax Fax" : "New fax from Pax Fax"
},
"nplurals=2; plural=(n != 1);");

